"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hebe = void 0;
const querystring_1 = require("querystring");
class Hebe {
    constructor({ username, password } = {}) {
        this.noTokenError = "Use hebe.authenticate() in order to obtain the token";
        this.baseUrl = "https://www.hebe.pl";
        this.authenticate = (...args_1) => __awaiter(this, [...args_1], void 0, function* ({ username, password, } = {}) {
            if (username) {
                this.username = username;
            }
            if (password) {
                this.password = password;
            }
            const { csrfToken, dwsidToken } = yield this.obtainInitialToken();
            this.token = yield this.obtainAuthDwsidToken(dwsidToken, csrfToken);
        });
        this.obtainAuthDwsidToken = (initialDwsidToken, csrfToken) => __awaiter(this, void 0, void 0, function* () {
            const url = `${this.baseUrl}/on/demandware.store/Sites-Hebe-Site/en_US/Login-LoginForm`;
            const headers = Object.assign(Object.assign({}, this.headers), { Cookie: `dwsid=${initialDwsidToken};` });
            const body = (0, querystring_1.stringify)({
                dwfrm_login_username_d0uexrwjfbur: this.username,
                dwfrm_login_password_d0vyhxgwmmwx: this.password,
                csrf_token: csrfToken,
                dwfrm_login_login: "",
                format: "ajax",
            });
            const response = yield fetch(url, {
                method: "POST",
                headers,
                body,
            });
            if (response.status !== 200) {
                throw new Error(`Authentication failed, status code: ${response.status}`);
            }
            const { error, status } = JSON.parse(yield response.text());
            if (error || status !== "SUCCESS") {
                throw new Error(`Authentication failed: ${error}. Status: ${status}`);
            }
            const dwsidCookie = response.headers
                .getSetCookie()
                .find((cookie) => cookie.includes("dwsid"));
            if (!dwsidCookie) {
                throw new Error("dwsid cookie is not present in the initial token retrieval");
            }
            const dwsidToken = dwsidCookie.split("dwsid=")[1].split(";")[0];
            if (!dwsidToken || typeof dwsidToken !== "string") {
                throw new Error("dwsidToken is not present in the initial token retrieval");
            }
            return dwsidToken;
        });
        this.obtainInitialToken = () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const response = yield fetch(this.baseUrl);
            const dwsidCookie = response.headers
                .getSetCookie()
                .find((cookie) => cookie.includes("dwsid"));
            const txt = yield response.text();
            if (!dwsidCookie) {
                throw new Error("dwsid cookie is not present in the initial token retrieval");
            }
            const dwsidToken = dwsidCookie.split("dwsid=")[1].split(";")[0];
            const csrfRaw = (_a = txt === null || txt === void 0 ? void 0 : txt.split("window.CSRFToken = ")[1]) === null || _a === void 0 ? void 0 : _a.split(";")[0];
            if (!csrfRaw) {
                throw new Error("csrfToken is not present in the initial token retrieval");
            }
            const csrfJson = JSON.parse(csrfRaw);
            const csrfToken = csrfJson === null || csrfJson === void 0 ? void 0 : csrfJson.value;
            if (!csrfToken || typeof csrfToken !== "string") {
                throw new Error("csrfToken is not present in the initial token retrieval");
            }
            return {
                csrfToken,
                dwsidToken,
            };
        });
        this.username = username !== null && username !== void 0 ? username : "";
        this.password = password !== null && password !== void 0 ? password : "";
    }
    get headers() {
        return Object.assign({ "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8", "X-Requested-With": "XMLHttpRequest", Origin: this.baseUrl, Referer: `${this.baseUrl}/home?showform=true&targeturl=%2Faccount` }, (this.token && {
            Cookie: `dwsid=${this.token}`,
        }));
    }
}
exports.Hebe = Hebe;
